@@ -57,7 +57,9 @@ VL char const n_month_names[12 + 1][4] = {
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", ""
 };
 VL char const n_uagent[sizeof VAL_UAGENT] = VAL_UAGENT;
+#ifdef HAVE_UISTRINGS
 VL char const n_error[sizeof n_ERROR] = N_(n_ERROR);
+#endif
 VL char const n_path_devnull[sizeof n_PATH_DEVNULL] = n_PATH_DEVNULL;
 VL char const n_reproducible_name[sizeof "reproducible_build"] =
       "reproducible_build";
@@ -244,7 +246,7 @@ a_main_usage(FILE *fp){
       n_progname, n_uagent, ok_vlook(version));
    fprintf(fp, _(
       "Send-only mode: send mail \"to-address\" receiver(s):\n"
-      "  %s [-BdEFinv~#] [-: spec] [-A account] [:-C \"custom: header\":]\n"
+      "  %s [-BDdEFinv~#] [-: spec] [-A account] [:-C \"custom: header\":]\n"
       "  %s [:-a attachment:] [:-b bcc-address:]\n"
       "  %s [:-c cc-address:] [-M type | -m file | -q file | -t]\n"
       "  %s [-r from-address] [:-S var[=value]:] [-s subject] [:-X cmd:]\n"
@@ -252,13 +254,13 @@ a_main_usage(FILE *fp){
       n_progname, buf, buf, buf, buf);
    fprintf(fp, _(
       "\"Receive\" mode, starting on -u user, primary *inbox* or [$MAIL]:\n"
-      "  %s [-BdEeHiNnRv~#] [-: spec] [-A account] [:-C \"custom: header\":]\n"
+     "  %s [-BDdEeHiNnRv~#] [-: spec] [-A account] [:-C \"custom: header\":]\n"
       "  %s [-L spec] [-r from-address] [:-S var[=value]:]\n"
       "  %s [-u user] [:-X cmd:] [-- :mta-option:]\n"),
       n_progname, buf, buf);
    fprintf(fp, _(
       "\"Receive\" mode, starting on -f (secondary $MBOX or [file]):\n"
-      "  %s [-BdEeHiNnRv~#] [-: spec] [-A account] [:-C \"custom: header\":]\n"
+     "  %s [-BDdEeHiNnRv~#] [-: spec] [-A account] [:-C \"custom: header\":]\n"
       "  %s -f [-L spec] [-r from-address] [:-S var[=value]:]\n"
       "  %s [:-X cmd:] [file] [-- :mta-option:]\n"),
       n_progname, buf, buf);
@@ -648,7 +650,7 @@ main(int argc, char *argv[]){
     * TODO Like so we can get rid of some stack locals etc. */
    /* Keep in SYNC: ./nail.1:"SYNOPSIS, main() */
    static char const optstr[] =
-         "A:a:Bb:C:c:dEeFfHhiL:M:m:NnO:q:Rr:S:s:tu:VvX:::~#.";
+         "A:a:Bb:C:c:DdEeFfHhiL:M:m:NnO:q:Rr:S:s:tu:VvX:::~#.";
    int i;
    char *cp;
    enum{
@@ -733,11 +735,15 @@ main(int argc, char *argv[]){
          n_psonce |= n_PSO_SENDMODE;
          cc = cat(cc, lextract(a_main_oarg, GCC | GFULL));
          break;
+      case 'D':
+#ifdef HAVE_IMAP
+         ok_bset(disconnected);
+#endif
+         break;
       case 'd':
          ok_bset(debug);
          break;
       case 'E':
-         n_OBSOLETE(_("-E will be removed, please use \"-Sskipemptybody\""));
          ok_bset(skipemptybody);
          break;
       case 'e':
@@ -1105,7 +1111,7 @@ jgetopt_done:
             n_OBSOLETE(_("Please use $MAILX_NO_SYSTEM_RC instead of "
                "$NAIL_NO_SYSTEM_RC"));
          if(!nload && !ok_blook(MAILX_NO_SYSTEM_RC) &&
-               !n_go_load(VAL_SYSCONFDIR "/" VAL_SYSCONFRC))
+               !n_go_load(ok_vlook(system_mailrc)))
             goto j_leave;
       }
 
@@ -1166,7 +1172,11 @@ je_expandargv:
 
       a[0] = Aarg;
       a[1] = NULL;
-      c_account(a);
+      if(c_account(a) && (!(n_psonce & n_PSO_INTERACTIVE) ||
+            ok_blook(errexit) || ok_blook(posix))){
+         n_exit_status = n_EXIT_USE | n_EXIT_SEND_ERROR;
+         goto jleave;
+      }
    }
 
    /*
