@@ -22,7 +22,6 @@ XOPTIONS="\
    ICONV='Character set conversion using iconv(3)' \
    SOCKETS='Network support' \
       SSL='SSL/TLS (OpenSSL / LibreSSL)' \
-         SSL_RANDOM='-' \
          SSL_ALL_ALGORITHMS='Support of all digest and cipher algorithms' \
       SMTP='Simple Mail Transfer Protocol client' \
       POP3='Post Office Protocol Version 3 client' \
@@ -39,11 +38,12 @@ XOPTIONS="\
       KEY_BINDINGS='Configurable key bindings' \
    TERMCAP='Terminal capability queries (termcap(5))' \
       TERMCAP_VIA_TERMINFO='Terminal capability queries use terminfo(5)' \
-   ERRORS='Error log message ring' \
+   ERRORS='Log message ring' \
    SPAM_SPAMC='Spam management via spamc(1) of spamassassin(1)' \
    SPAM_SPAMD='-' \
    SPAM_FILTER='Freely configurable *spam-filter-..*s' \
    DOCSTRINGS='Command documentation help strings' \
+   UISTRINGS='User interface and error message strings' \
    QUOTE_FOLD='Extended *quote-fold*ing' \
    FILTER_HTML_TAGSOUP='Simple built-in HTML-to-text display filter' \
    COLOUR='Coloured message display' \
@@ -70,6 +70,15 @@ XOPTIONS_XTRA="\
    DEVEL='Computers do not blunder' \
 "
 
+# To avoid too many recompilations we use a two-stage "configuration changed"
+# detection, the first uses mk-config.lst, which only goes for actual user
+# config settings etc. the second uses mk-config.h, which thus includes the
+# things we have truly detected.  This does not work well for multiple choice
+# values of which only one will be really used, so those user wishes may not be
+# placed in the header, only the really detected one (but that has to!).
+# Used for grep(1), for portability assume fixed matching only.
+H_BLACKLIST='-e VAL_RANDOM -e VAL_IDNA'
+
 # The problem is that we don't have any tools we can use right now, so
 # encapsulate stuff in functions which get called in right order later on
 
@@ -104,11 +113,13 @@ option_setup() {
          ;;
       [nN][uU][lL][lL][iI])
          OPT_ICONV=require
+         OPT_UISTRINGS=1
          ;;
       [mM][iI][nN][iI][mM][aA][lL])
          OPT_DOTLOCK=require OPT_ICONV=require OPT_REGEX=require
          OPT_COLOUR=1
          OPT_DOCSTRINGS=1
+         OPT_UISTRINGS=1
          OPT_ERRORS=1
          OPT_IDNA=1
          OPT_MLE=1
@@ -124,6 +135,7 @@ option_setup() {
                OPT_AGENT=1
          OPT_COLOUR=1
          OPT_DOCSTRINGS=1
+         OPT_UISTRINGS=1
          OPT_ERRORS=1
          OPT_IDNA=1
          OPT_MLE=1
@@ -153,12 +165,8 @@ option_setup() {
 
 # Inter-relationships XXX sort this!
 option_update() {
-   if feat_yes NOEXTRANDOM; then
-      OPT_SSL_RANDOM=0
-   fi
-
    if feat_no SSL; then
-      OPT_SSL_RANDOM=0 OPT_SSL_ALL_ALGORITHMS=0
+      OPT_SSL_ALL_ALGORITHMS=0
    fi
 
    if feat_no SMTP && feat_no POP3 && feat_no IMAP; then
@@ -177,7 +185,7 @@ option_update() {
          msg 'ERROR: need SOCKETS for required feature IMAP'
          config_exit 13
       fi
-      OPT_SSL=0 OPT_SSL_RANDOM=0 OPT_SSL_ALL_ALGORITHMS=0
+      OPT_SSL=0 OPT_SSL_ALL_ALGORITHMS=0
       OPT_SMTP=0 OPT_POP3=0 OPT_IMAP=0
       OPT_GSSAPI=0 OPT_NETRC=0 OPT_AGENT=0
    fi
@@ -234,8 +242,10 @@ mk=./mk-config.mk
 
 newlst=./mk-nconfig.lst
 newmk=./mk-nconfig.mk
+oldmk=./mk-oconfig.mk
 newev=./mk-nconfig.ev
 newh=./mk-nconfig.h
+oldh=./mk-oconfig.h
 tmp0=___tmp
 tmp=./${tmp0}1$$
 tmp2=./${tmp0}2$$
@@ -256,6 +266,9 @@ tmp2=./${tmp0}2$$
 _CFLAGS= _LDFLAGS=
 
 os_early_setup() {
+   [ -n "${OS}" ] && [ -n "${OSENV}" ] && [ -n "${OSFULLSPEC}" ] ||
+      thecmd_testandset_fail uname uname
+
    # We don't "have any utility": only path adjustments and such in here!
    [ -n "${OS}" ] || OS=`${uname} -s`
    export OS
@@ -445,7 +458,6 @@ cc_flags() {
          CFLAGS="-DNDEBUG ${CFLAGS}"
       fi
    fi
-   msg ''
    export CFLAGS LDFLAGS
 }
 
@@ -453,6 +465,7 @@ _cc_flags_tcc() {
    __cflags=${_CFLAGS} __ldflags=${_LDFLAGS}
    _CFLAGS= _LDFLAGS=
 
+   cc_check -W
    cc_check -Wall
    cc_check -Wextra
    cc_check -pedantic
@@ -508,6 +521,7 @@ _cc_flags_generic() {
    #if feat_yes DEVEL && cc_check -Weverything; then
    #   :
    #else
+      cc_check -W
       cc_check -Wall
       cc_check -Wextra
       cc_check -Wbad-function-cast
@@ -760,10 +774,31 @@ feat_bail_required() {
       msg 'ERROR: feature OPT_%s is required but not available' "${1}"
       config_exit 13
    fi
+   feat_is_unsupported "${1}"
+}
+
+feat_is_disabled() {
+   [ ${#} -eq 1 ] && msg ' .  (disabled: OPT_%s)' "${1}"
+   echo "/* OPT_${1} -> HAVE_${1} */" >> ${h}
+}
+
+feat_is_unsupported() {
+   msg ' ! NOTICE: unsupported: OPT_%s' "${1}"
+   echo "/* OPT_${1} -> HAVE_${1} */" >> ${h}
    eval OPT_${1}=0
    option_update # XXX this is rather useless here (dependency chain..)
 }
 
+feat_def() {
+   if feat_yes ${1}; then
+      echo '#define HAVE_'${1}'' >> ${h}
+      return 0
+   else
+      feat_is_disabled "${@}"
+      return 1
+   fi
+}
+
 option_parse() {
    # Parse one of our XOPTIONS* in $2 and assign the sh(1) compatible list of
    # options, without documentation, to $1
@@ -906,6 +941,8 @@ option_evaluate() {
             msg 'ERROR: cannot parse <%s>' "${line}"
             config_exit 1
          fi
+      elif { echo ${i} | ${grep} ${H_BLACKLIST} >/dev/null 2>&1; }; then
+         :
       else
          printf "#define ${i} \"${j}\"\n" >> ${newh}
       fi
@@ -916,6 +953,48 @@ option_evaluate() {
    exec 0<&5 1>&6 5<&- 6<&-
 }
 
+val_allof() {
+   eval __expo__=\$${1}
+   ${awk} -v HEAP="${2}" -v USER="${__expo__}" '
+      BEGIN{
+         i = split(HEAP, ha)
+         if((j = split(USER, ua)) == 0)
+            exit
+         for(; j != 0; --j){
+            us = tolower(ua[j])
+            if(us == "all" || us == "any")
+               continue
+            ok = 0
+            for(ii = i; ii != 0; --ii)
+               if(tolower(ha[ii]) == us){
+                  ok = 1
+                  break
+               }
+            if(!ok)
+               exit 1
+         }
+      }
+   '
+   __rv__=${?}
+   [ ${__rv__} -ne 0 ] && return ${__rv__}
+
+    if ${awk} -v USER="${__expo__}" '
+            BEGIN{
+               if((j = split(USER, ua)) == 0)
+                  exit
+               for(; j != 0; --j){
+                  us = tolower(ua[j])
+                  if(us == "all" || us == "any")
+                     exit 0
+               }
+               exit 1
+            }
+         '; then
+      eval "${1}"=\"${2}\"
+   fi
+   return 0
+}
+
 path_check() {
    # "path_check VARNAME" or "path_check VARNAME FLAG VARNAME"
    varname=${1} addflag=${2} flagvarname=${3}
@@ -1008,7 +1087,7 @@ _check_preface() {
 
    echo '**********'
    msg_nonl ' . %s ... ' "${topic}"
-   echo "/* checked ${topic} */" >> ${h}
+   #echo "/* checked ${topic} */" >> ${h}
    ${rm} -f ${tmp} ${tmp}.o
    if [ "${dump_test_program}" = 1 ]; then
       echo '*** test program is'
@@ -1042,7 +1121,7 @@ without_check() {
       eval have_${variable}=yes
       return 0
    else
-      echo "/* ${define} */" >> ${h}
+      #echo "/* ${define} */" >> ${h}
       msg 'no (deduced)'
       eval unset have_${variable}
       return 1
@@ -1063,7 +1142,7 @@ compile_check() {
       eval have_${variable}=yes
       return 0
    else
-      echo "/* ${define} */" >> ${h}
+      #echo "/* ${define} */" >> ${h}
       msg 'no'
       eval unset have_${variable}
       return 1
@@ -1098,7 +1177,7 @@ _link_mayrun() {
       return 0
    else
       msg 'no'
-      echo "/* ${define} */" >> ${h}
+      #echo "/* ${define} */" >> ${h}
       eval unset have_${variable}
       return 1
    fi
@@ -1116,16 +1195,6 @@ xrun_check() {
    _link_mayrun 2 "${1}" "${2}" "${3}" "${4}" "${5}"
 }
 
-feat_def() {
-   if feat_yes ${1}; then
-      echo '#define HAVE_'${1}'' >> ${h}
-      return 0
-   else
-      echo '/* OPT_'${1}'=0 */' >> ${h}
-      return 1
-   fi
-}
-
 squeeze_em() {
    < "${1}" > "${2}" ${awk} \
    'BEGIN {ORS = " "} /^[^#]/ {print} {next} END {ORS = ""; print "\n"}'
@@ -1137,7 +1206,6 @@ squeeze_em() {
 
 # Very easy checks for the operating system in order to be able to adjust paths
 # or similar very basic things which we need to be able to go at all
-thecmd_testandset_fail uname uname
 os_early_setup
 
 # Check those tools right now that we need before including $rc
@@ -1202,7 +1270,8 @@ option_update
 # (No functions since some shells loose non-exported variables in traps)
 trap "trap \"\" HUP INT TERM; exit 1" HUP INT TERM
 trap "trap \"\" HUP INT TERM EXIT;\
-   ${rm} -rf ${newlst} ${tmp0}.* ${tmp0}* ${newmk} ${newev} ${newh}" EXIT
+   ${rm} -rf ${newlst} ${tmp0}.* ${tmp0}* \
+      ${newmk} ${oldmk} ${newev} ${newh} ${oldh}" EXIT
 
 # Our configuration options may at this point still contain shell snippets,
 # we need to evaluate them in order to get them expanded, and we need those
@@ -1323,13 +1392,13 @@ done
 
 # Now finally check whether we already have a configuration and if so, whether
 # all those parameters are still the same.. or something has actually changed
+config_updated=
 if [ -f ${lst} ] && ${cmp} ${newlst} ${lst} >/dev/null 2>&1; then
    echo 'Configuration is up-to-date'
    exit 0
 elif [ -f ${lst} ]; then
+   config_updated=1
    echo 'Configuration has been updated..'
-   ( eval "${MAKE} -f ./mk-config.mk clean" )
-   echo
 else
    echo 'Shiny configuration..'
 fi
@@ -1342,7 +1411,9 @@ config_exit() {
 
 ${mv} -f ${newlst} ${lst}
 ${mv} -f ${newev} ${ev}
+[ -f ${h} ] && ${mv} -f ${h} ${oldh}
 ${mv} -f ${newh} ${h}
+[ -f ${mk} ] && ${mv} -f ${mk} ${oldmk}
 ${mv} -f ${newmk} ${mk}
 
 ## Compile and link checking
@@ -1355,9 +1426,10 @@ makefile=./${tmp0}.mk
 
 # (No function since some shells loose non-exported variables in traps)
 trap "trap \"\" HUP INT TERM;\
-   ${rm} -f ${lst} ${h} ${mk} ${lib} ${inc}; exit 1" HUP INT TERM
+   ${rm} -f ${lst} ${oldh} ${h} ${oldmk} ${mk} ${lib} ${inc}; exit 1" \
+      HUP INT TERM
 trap "trap \"\" HUP INT TERM EXIT;\
-   ${rm} -rf ${tmp0}.* ${tmp0}*" EXIT
+   ${rm} -rf ${oldh} ${oldmk} ${tmp0}.* ${tmp0}*" EXIT
 
 # Time to redefine helper 2
 msg() {
@@ -1406,17 +1478,18 @@ dump_test_program=0
 dump_test_program=1
 
 feat_def ALWAYS_UNICODE_LOCALE
-feat_def AMALGAMATION
+feat_def AMALGAMATION 0
 feat_def CROSS_BUILD
 feat_def DOCSTRINGS
+feat_def UISTRINGS
 feat_def ERRORS
 
-feat_def ASAN_ADDRESS
-feat_def ASAN_MEMORY
-feat_def DEBUG
-feat_def DEVEL
-feat_def NYD2
-feat_def NOMEMDBG
+feat_def ASAN_ADDRESS 0
+feat_def ASAN_MEMORY 0
+feat_def DEBUG 0
+feat_def DEVEL 0
+feat_def NYD2 0
+feat_def NOMEMDBG 0
 
 if xrun_check inline 'inline functions' \
    '#define HAVE_INLINE
@@ -1576,6 +1649,7 @@ int main(void){
 then
    :
 else
+   # TODO support HAVE_FTRUNCATE *everywhere*, do not require this syscall!
    msg 'ERROR: we require the ftruncate(2) system call.'
    config_exit 1
 fi
@@ -1793,7 +1867,8 @@ int main(void){
 
 ##
 
-# The random check has been moved to below SSL detection due to OPT_SSL_RANDOM
+# The random check has been moved to below SSL detection due to multiple choice
+# selection for PRG sources
 
 link_check putc_unlocked 'putc_unlocked(3)' '#define HAVE_PUTC_UNLOCKED' <<\!
 #include <stdio.h>
@@ -1836,7 +1911,9 @@ int main(void){
    fi
 fi
 
+##
 ## optional and selectable
+##
 
 if feat_yes DOTLOCK; then
    if run_check readlink 'readlink(2)' << \!
@@ -2064,7 +2141,7 @@ int main(void){
       esac
    fi
 else
-   echo '/* OPT_ICONV=0 */' >> ${h}
+   feat_is_disabled ICONV
 fi # feat_yes ICONV
 
 if feat_yes SOCKETS || feat_yes SPAM_SPAMD; then
@@ -2119,7 +2196,7 @@ int main(void){
          '#define HAVE_SOCKETS' '-lsocket -lnsl' ||
       feat_bail_required SOCKETS
 else
-   echo '/* OPT_SOCKETS=0 */' >> ${h}
+   feat_is_disabled SOCKETS
 fi # feat_yes SOCKETS
 
 if feat_yes SOCKETS; then
@@ -2544,14 +2621,15 @@ int main(void){
 }
 !
    fi # }}}
-fi
-if feat_yes SSL; then
-   feat_def SSL_RANDOM
-   feat_def SSL_ALL_ALGORITHMS
+
+   if feat_yes SSL; then
+      feat_def SSL_ALL_ALGORITHMS
+   else
+      feat_bail_required SSL_ALL_ALGORITHMS
+   fi
 else
-   feat_bail_required SSL_RANDOM
-   feat_bail_required SSL_ALL_ALGORITHMS
-   echo '/* OPT_SSL=0 */' >> ${h}
+   feat_is_disabled SSL
+   feat_is_disabled SSL_ALL_ALGORITHMS
 fi # }}} feat_yes SSL
 printf '#define VAL_SSL_FEATURES "#'"${VAL_SSL_FEATURES}"'"\n' >> ${h}
 
@@ -2562,24 +2640,42 @@ else
 fi
 feat_def SMIME
 
-# Native random check (had been delayed due to OPT_SSL_RAMDOM) {{{
-# XXX Add POSIX check once standardized
-if feat_yes NOEXTRANDOM; then
-   echo '#define HAVE_NOEXTRANDOM' >> ${h}
-elif feat_yes SSL_RANDOM; then
+# VAL_RANDOM {{{
+if val_allof VAL_RANDOM \
+      "arc4 ssl libgetrandom sysgetrandom urandom builtin error"; then
    :
-elif link_check arc4random 'arc4random(3)' '#define HAVE_POSIX_RANDOM 0' << \!
+else
+   msg 'ERROR: VAL_RANDOM with invalid entries: %s' "${VAL_RANDOM}"
+   config_exit 1
+fi
+
+val_random_arc4() {
+   link_check arc4random 'VAL_RANDOM: arc4random(3)' \
+      '#define HAVE_RANDOM n_RANDOM_IMPL_ARC4' << \!
 #include <stdlib.h>
 int main(void){
    arc4random();
    return 0;
 }
 !
-then
-   :
-elif link_check getrandom 'getrandom(2) (in sys/random.h)' \
-      '#define HAVE_GETRANDOM(B,S) getrandom(B, S, 0)
-      #define HAVE_GETRANDOM_HEADER <sys/random.h>' <<\!
+}
+
+val_random_ssl() {
+   if feat_yes SSL; then
+      msg ' . VAL_RANDOM: ssl ... yes'
+      echo '#define HAVE_RANDOM n_RANDOM_IMPL_SSL' >> ${h}
+      return 0
+   else
+      msg ' . VAL_RANDOM: ssl ... no'
+      return 1
+   fi
+}
+
+val_random_libgetrandom() {
+   link_check getrandom 'VAL_RANDOM: getrandom(3) (in sys/random.h)' \
+      '#define HAVE_RANDOM n_RANDOM_IMPL_GETRANDOM
+      #define n_RANDOM_GETRANDOM_FUN(B,S) getrandom(B, S, 0)
+      #define n_RANDOM_GETRANDOM_H <sys/random.h>' <<\!
 #include <sys/random.h>
 int main(void){
    char buf[256];
@@ -2587,11 +2683,13 @@ int main(void){
    return 0;
 }
 !
-then
-   :
-elif link_check getrandom 'getrandom(2) (via syscall(2))' \
-      '#define HAVE_GETRANDOM(B,S) syscall(SYS_getrandom, B, S, 0)
-      #define HAVE_GETRANDOM_HEADER <sys/syscall.h>' <<\!
+}
+
+val_random_sysgetrandom() {
+   link_check getrandom 'VAL_RANDOM: getrandom(2) (via syscall(2))' \
+      '#define HAVE_RANDOM n_RANDOM_IMPL_GETRANDOM
+      #define n_RANDOM_GETRANDOM_FUN(B,S) syscall(SYS_getrandom, B, S, 0)
+      #define n_RANDOM_GETRANDOM_H <sys/syscall.h>' <<\!
 #include <sys/syscall.h>
 int main(void){
    char buf[256];
@@ -2599,13 +2697,50 @@ int main(void){
    return 0;
 }
 !
-then
-   :
-elif [ -n "${have_no_subsecond_time}" ]; then
-   msg 'ERROR: %s %s' 'without a native random' \
-      'one of clock_gettime(2) and gettimeofday(2) is required.'
-   config_exit 1
-fi # }}}
+}
+
+val_random_urandom() {
+   msg_nonl ' . VAL_RANDOM: /dev/urandom ... '
+   if feat_yes CROSS_BUILD; then
+      msg 'yes (unchecked)'
+      echo '#define HAVE_RANDOM n_RANDOM_IMPL_URANDOM' >> ${h}
+   elif [ -f /dev/urandom ]; then
+      msg yes
+      echo '#define HAVE_RANDOM n_RANDOM_IMPL_URANDOM' >> ${h}
+   else
+      msg no
+      return 1
+   fi
+   return 0
+}
+
+val_random_builtin() {
+   msg_nonl ' . VAL_RANDOM: builtin ... '
+   if [ -n "${have_no_subsecond_time}" ]; then
+      msg 'no\nERROR: %s %s' 'without a specialized PRG ' \
+         'one of clock_gettime(2) and gettimeofday(2) is required.'
+      config_exit 1
+   else
+      msg yes
+      echo '#define HAVE_RANDOM n_RANDOM_IMPL_BUILTIN' >> ${h}
+   fi
+}
+
+val_random_error() {
+   msg 'ERROR: VAL_RANDOM search reached "error" entry'
+   config_exit 42
+}
+
+oifs=${IFS}
+unset IFS
+VAL_RANDOM="${VAL_RANDOM} error"
+set -- ${VAL_RANDOM}
+IFS=${oifs}
+for randfun
+do
+   eval val_random_$randfun && break
+done
+# }}} VAL_RANDOM
 
 feat_def SMTP
 feat_def POP3
@@ -2668,15 +2803,23 @@ int main(void){
       feat_bail_required GSSAPI
    fi
 else
-   echo '/* OPT_GSSAPI=0 */' >> ${h}
+   feat_is_disabled GSSAPI
 fi # feat_yes GSSAPI
 
 feat_def NETRC
 feat_def AGENT
 
-if feat_yes IDNA; then
-   if link_check idna 'Libidn2' '#define HAVE_IDNA HAVE_IDNA_LIBIDN2' \
-         '-lidn2' << \!
+if feat_yes IDNA; then # {{{
+   if val_allof VAL_IDNA "idnkit idn2 idn"; then
+      :
+   else
+      msg 'ERROR: VAL_IDNA with invalid entries: %s' "${VAL_IDNA}"
+      config_exit 1
+   fi
+
+   val_idna_idn2() {
+      link_check idna 'OPT_IDNA: GNU Libidn2' \
+         '#define HAVE_IDNA n_IDNA_IMPL_LIBIDN2' '-lidn2' << \!
 #include <idn2.h>
 int main(void){
    char *idna_utf8, *idna_lc;
@@ -2691,10 +2834,11 @@ int main(void){
    return 0;
 }
 !
-   then
-      :
-   elif link_check idna 'GNU Libidn' '#define HAVE_IDNA HAVE_IDNA_LIBIDNA' \
-         '-lidn' << \!
+   }
+
+   val_idna_idn() {
+      link_check idna 'OPT_IDNA: GNU Libidn' \
+         '#define HAVE_IDNA n_IDNA_IMPL_LIBIDN' '-lidn' << \!
 #include <idna.h>
 #include <idn-free.h>
 #include <stringprep.h> /* XXX we actually use our own iconv instead */
@@ -2711,10 +2855,11 @@ int main(void){
    return 0;
 }
 !
-   then
-      :
-   elif link_check idna 'idnkit' '#define HAVE_IDNA HAVE_IDNA_IDNKIT' \
-         '-lidnkit' << \!
+   }
+
+   val_idna_idnkit() {
+      link_check idna 'OPT_IDNA: idnkit' \
+         '#define HAVE_IDNA n_IDNA_IMPL_IDNKIT' '-lidnkit' << \!
 #include <stdio.h>
 #include <idn/api.h>
 #include <idn/result.h>
@@ -2737,20 +2882,24 @@ int main(void){
    return 0;
 }
 !
-   then
-      :
-   else
-      feat_bail_required IDNA
-   fi
+   }
 
-   if [ -n "${have_idna}" ]; then
-      echo '#define HAVE_IDNA_LIBIDN2 0' >> ${h}
-      echo '#define HAVE_IDNA_LIBIDNA 1' >> ${h}
-      echo '#define HAVE_IDNA_IDNKIT 2' >> ${h}
-   fi
+   val_idna_bye() {
+      feat_bail_required IDNA
+   }
+
+   oifs=${IFS}
+   unset IFS
+   VAL_IDNA="${VAL_IDNA} bye"
+   set -- ${VAL_IDNA}
+   IFS=${oifs}
+   for randfun
+   do
+      eval val_idna_$randfun && break
+   done
 else
-   echo '/* OPT_IDNA=0 */' >> ${h}
-fi
+   feat_is_disabled IDNA
+fi # }}} IDNA
 
 feat_def IMAP_SEARCH
 
@@ -2776,15 +2925,18 @@ int main(void){
       feat_bail_required REGEX
    fi
 else
-   echo '/* OPT_REGEX=0 */' >> ${h}
+   feat_is_disabled REGEX
 fi
 
-if feat_yes MLE && [ -n "${have_c90amend1}" ]; then
-   have_mle=1
-   echo '#define HAVE_MLE' >> ${h}
+if feat_yes MLE; then
+   if [ -n "${have_c90amend1}" ]; then
+      have_mle=1
+      echo '#define HAVE_MLE' >> ${h}
+   else
+      feat_bail_required MLE
+   fi
 else
-   feat_bail_required MLE
-   echo '/* OPT_MLE=0 */' >> ${h}
+   feat_is_disabled MLE
 fi
 
 # Generic have-a-line-editor switch for those who need it below
@@ -2792,16 +2944,24 @@ if [ -n "${have_mle}" ]; then
    have_cle=1
 fi
 
-if [ -n "${have_cle}" ] && feat_yes HISTORY; then
-   echo '#define HAVE_HISTORY' >> ${h}
+if feat_yes HISTORY; then
+   if [ -n "${have_cle}" ]; then
+      echo '#define HAVE_HISTORY' >> ${h}
+   else
+      feat_is_unsupported HISTORY
+   fi
 else
-   echo '/* OPT_HISTORY=0 */' >> ${h}
+   feat_is_disabled HISTORY
 fi
 
-if [ -n "${have_mle}" ] && feat_yes KEY_BINDINGS; then
-   echo '#define HAVE_KEY_BINDINGS' >> ${h}
+if feat_yes KEY_BINDINGS; then
+   if [ -n "${have_mle}" ]; then
+      echo '#define HAVE_KEY_BINDINGS' >> ${h}
+   else
+      feat_is_unsupported KEY_BINDINGS
+   fi
 else
-   echo '/* OPT_KEY_BINDINGS=0 */' >> ${h}
+   feat_is_disabled KEY_BINDINGS
 fi
 
 if feat_yes TERMCAP; then
@@ -2895,8 +3055,8 @@ _EOT
       fi
    fi
 else
-   echo '/* OPT_TERMCAP=0 */' >> ${h}
-   echo '/* OPT_TERMCAP_VIA_TERMINFO=0 */' >> ${h}
+   feat_is_disabled TERMCAP
+   feat_is_disabled TERMCAP_VIA_TERMINFO
 fi
 
 if feat_def SPAM_SPAMC; then
@@ -2905,11 +3065,14 @@ if feat_def SPAM_SPAMC; then
    fi
 fi
 
-if feat_yes SPAM_SPAMD && [ -n "${have_af_unix}" ]; then
-   echo '#define HAVE_SPAM_SPAMD' >> ${h}
+if feat_yes SPAM_SPAMD; then
+   if [ -n "${have_af_unix}" ]; then
+      echo '#define HAVE_SPAM_SPAMD' >> ${h}
+   else
+      feat_bail_required SPAM_SPAMD
+   fi
 else
-   feat_bail_required SPAM_SPAMD
-   echo '/* OPT_SPAM_SPAMD=0 */' >> ${h}
+   feat_is_disabled SPAM_SPAMD
 fi
 
 feat_def SPAM_FILTER
@@ -2920,12 +3083,14 @@ else
    echo '/* HAVE_SPAM */' >> ${h}
 fi
 
-if feat_yes QUOTE_FOLD &&\
-      [ -n "${have_c90amend1}" ] && [ -n "${have_wcwidth}" ]; then
-   echo '#define HAVE_QUOTE_FOLD' >> ${h}
+if feat_yes QUOTE_FOLD; then
+   if [ -n "${have_c90amend1}" ] && [ -n "${have_wcwidth}" ]; then
+      echo '#define HAVE_QUOTE_FOLD' >> ${h}
+   else
+      feat_bail_required QUOTE_FOLD
+   fi
 else
-   feat_bail_required QUOTE_FOLD
-   echo '/* OPT_QUOTE_FOLD=0 */' >> ${h}
+   feat_is_disabled QUOTE_FOLD
 fi
 
 feat_def FILTER_HTML_TAGSOUP
@@ -2945,13 +3110,19 @@ ${mv} ${tmp} ${lib}
 ${mv} ${h} ${tmp}
 printf '#ifndef n_MK_CONFIG_H\n# define n_MK_CONFIG_H 1\n' > ${h}
 ${cat} ${tmp} >> ${h}
-${rm} -f ${tmp}
 printf '\n' >> ${h}
+# We need these for correct "second stage configuration changed" detection */
+echo "/* `${cat} ${lib}` */" >> ${h}
+echo "/* `${cat} ${inc}` */" >> ${h}
+printf '\n' >> ${h}
+
+# Throw away all temporaries
+${rm} -rf ${tmp0}.* ${tmp0}*
 
 # Create the string that is used by *features* and `version'.
 # Take this nice opportunity and generate a visual listing of included and
 # non-included features for the person who runs the configuration
-msg '\nThe following features are included (+) or not (-):'
+echo 'The following features are included (+) or not (-):' > ${tmp}
 set -- ${OPTIONS_DETECT} ${OPTIONS} ${OPTIONS_XTRA}
 printf '/* The "feature string" */\n' >> ${h}
 # Because + is expanded by *folder* if first in "echo $features", put something
@@ -2965,7 +3136,7 @@ do
    feat_yes ${opt} && sign=+ || sign=-
    printf -- "${sep}${sign}${sopt}" >> ${h}
    sep=','
-   msg " %s %s: %s" ${sign} ${sopt} "${sdoc}"
+   printf ' %s %s: %s\n' ${sign} ${sopt} "${sdoc}" >> ${tmp}
 done
 # TODO instead of using sh+tr+awk+printf, use awk, drop option_doc_of, inc here
 #exec 5>&1 >>${h}
@@ -2976,7 +3147,6 @@ printf '"\n' >> ${h}
 # Create the real mk-config.mk
 # Note we cannout use explicit ./ filename prefix for source and object
 # pathnames because of a bug in bmake(1)
-${rm} -rf ${tmp0}.* ${tmp0}*
 srclist= objlist=
 if feat_no AMALGAMATION; then
    for i in `printf '%s\n' "${SRCDIR}"*.c | ${sort}`; do
@@ -3022,7 +3192,28 @@ ${cat} "${SRCDIR}"make-config.in >> ${mk}
 
 ## Finished!
 
-msg '\nSetup:'
+# We have completed the new configuration header.  Check whether *really*
+# Do the "second stage configuration changed" detection, exit if nothing to do
+if [ -f ${oldh} ]; then
+   if ${cmp} ${h} ${oldh} >/dev/null 2>&1; then
+      ${mv} -f ${oldh} ${h}
+      msg 'Effective configuration is up-to-date'
+      exit 0
+   fi
+   config_updated=1
+   ${rm} -f ${oldh}
+   msg 'Effective configuration has been updated..'
+fi
+
+if [ -n "${config_updated}" ]; then
+   msg 'Wiping away old objects and such'
+   ( eval "${MAKE} -f ${oldmk} clean" )
+fi
+
+msg ''
+while read l; do msg "${l}"; done < ${tmp}
+
+msg 'Setup:'
 msg ' . System-wide resource file: %s/%s' "${VAL_SYSCONFDIR}" "${VAL_SYSCONFRC}"
 msg ' . bindir: %s' "${VAL_BINDIR}"
 if feat_yes DOTLOCK; then
@@ -3031,8 +3222,8 @@ fi
 msg ' . mandir: %s' "${VAL_MANDIR}"
 msg ' . M(ail)T(ransfer)A(gent): %s (argv0: %s)' "${VAL_MTA}" "${VAL_MTA_ARGV0}"
 msg ' . $MAIL spool directory: %s' "${VAL_MAIL}"
-msg ''
 
+msg ''
 if [ -n "${have_fnmatch}" ] && [ -n "${have_fchdir}" ]; then
    exit 0
 fi
